//#############################################################################
//
// FILE:   cmpss_ex1_asynch.c
//
// TITLE:  CMPSS Asynchronous Trip
//
//! \addtogroup driver_example_list
//! <h1> CMPSS Asynchronous Trip </h1>
//!
//! This example enables the CMPSS1 COMPH comparator and feeds the asynchronous
//! CTRIPOUTH signal to the GPIO14/OUTPUTXBAR3 pin and CTRIPH to GPIO15/EPWM8B.
//!
//! \b External \b Connections \n
//!  - Outputs can be observed on GPIO14 and GPIO15
//!  - Comparator input pin is on ADCINA2
//!
//! \b Watch \b Variables \n
//!  - None
//!
//
//#############################################################################
// $TI Release: F2837xD Support Library v3.03.00.00 $
// $Release Date: Thu Dec  7 18:51:32 CST 2017 $
// $Copyright:
// Copyright (C) 2013-2017 Texas Instruments Incorporated - http://www.ti.com/
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met:
// 
//   Redistributions of source code must retain the above copyright 
//   notice, this list of conditions and the following disclaimer.
// 
//   Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the 
//   documentation and/or other materials provided with the   
//   distribution.
// 
//   Neither the name of Texas Instruments Incorporated nor the names of
//   its contributors may be used to endorse or promote products derived
//   from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// $
//#############################################################################

//
// Included Files
//
#include "driverlib.h"
#include "device.h"


// 20MHz XTAL on controlCARD. For use with SysCtl_getClock().
#define DEVICE_OSCSRC_FREQ          20000000U

// Define to pass to SysCtl_setClock(). Will configure the clock as follows:
// PLLSYSCLK = 20MHz (XTAL_OSC) * 20 (IMULT) * 1 (FMULT) / 2 (PLLCLK_BY_2)
#define DEVICE_SETCLOCK_CFG         (SYSCTL_OSCSRC_XTAL | SYSCTL_IMULT(20) |  \
                                     SYSCTL_FMULT_NONE | SYSCTL_SYSDIV(2) |   \
                                     SYSCTL_PLL_ENABLE)

// 200MHz SYSCLK frequency based on the above DEVICE_SETCLOCK_CFG. Update the
// code below if a different clock configuration is used!
#define DEVICE_SYSCLK_FREQ          ((DEVICE_OSCSRC_FREQ * 20 * 1) / 2)

// 100MHz PWMCLK frequency based on the above DEVICE_SYSCLK_FREQ and a
// pwm clock divider of 2. Update the code below if a
// different EPWMCLKDIV divider is used!
#define DEVICE_PWMCLK_FREQ          (DEVICE_SYSCLK_FREQ / 2)

// Macro to call SysCtl_delay() to achieve a delay in microseconds. The macro
// will convert the desired delay in microseconds to the count value expected
// by the function. \b x is the number of microseconds to delay.
#define DEVICE_DELAY_US(x) SysCtl_delay(((((long double)(x)) / (1000000.0L /  \
                              (long double)DEVICE_SYSCLK_FREQ)) - 9.0L) / 5.0L)

#define PWM_FREQ 96000
#define PWM_DEAD_BAND_NS 150

#define PWM_TIMER_TBPRD  (int)((DEVICE_PWMCLK_FREQ/PWM_FREQ))
#define PWM_TIMER_CMP  PWM_TIMER_TBPRD/2
//#define PWM_DB (int)(PWM_DEAD_BAND_NS/(2000000000.0L/DEVICE_SYSCLK_FREQ))
#define PWM_DB 100

#define DVID 10 // 1000V/s : 1000V / 96kHz

#define RAMP_DECREMENT 20
#define RAMP_DELAY 0

float rampstart = 0.0;
float rampstartref = 0.0;

static inline void InitPWMGPIO(void);
static inline void InitEPWM1(void);
static inline void InitCMP1(void);
__interrupt void ISRPreControlTask(void);

//
// Main
//
void main(void)
{
    //
    // Initialize device clock and peripherals
    //
    Device_init();

    //
    // Disable pin locks and enable internal pullups.
    //
    Device_initGPIO();

    //
    // Initialize PIE and clear PIE registers. Disables CPU interrupts.
    //
    Interrupt_initModule();

    //
    // Initialize the PIE vector table with pointers to the shell Interrupt
    // Service Routines (ISR).
    //
    Interrupt_initVectorTable();

    // Disable sync(Freeze clock to PWM as well)
    SysCtl_disablePeripheral(SYSCTL_PERIPH_CLK_TBCLKSYNC);

    InitPWMGPIO();
    InitEPWM1();

    // Enable sync and clock to PWM
    SysCtl_enablePeripheral(SYSCTL_PERIPH_CLK_TBCLKSYNC);

    InitCMP1(); // Buck Phase 1

    //
    // Enable Global Interrupt (INTM) and realtime interrupt (DBGM)
    //
    EINT;
    ERTM;

    //
    // Loop indefinitely
    //
    while(1)
    {

    }
}





static inline void InitPWMGPIO(void)
{
  // GPIO4 is the EPWM3A pin.
  GPIO_setMasterCore(0, GPIO_CORE_CPU1);
  GPIO_setPinConfig(GPIO_0_EPWM1A);
  GPIO_setDirectionMode(0, GPIO_DIR_MODE_OUT);
  GPIO_setPadConfig(0, GPIO_PIN_TYPE_STD);

  // GPIO5 is the GPIO pin.
  GPIO_setMasterCore(1, GPIO_CORE_CPU1);
  GPIO_setPinConfig(GPIO_1_EPWM1B);
  GPIO_setDirectionMode(1, GPIO_DIR_MODE_OUT);
  GPIO_setPadConfig(1, GPIO_PIN_TYPE_STD);

  // GPIO8 is the EPWM5A pin.
  GPIO_setMasterCore(8, GPIO_CORE_CPU1);
  GPIO_setPinConfig(GPIO_8_EPWM5A);
  GPIO_setDirectionMode(8, GPIO_DIR_MODE_OUT);
  GPIO_setPadConfig(8, GPIO_PIN_TYPE_STD);
}

static inline void InitEPWM1(void)
{
  // Set-up TBCLK
  EPWM_setTimeBasePeriod(EPWM1_BASE, PWM_TIMER_TBPRD);
  //EPWM_setPhaseShift(EPWM3_BASE, 0);
  //EPWM_enablePhaseShiftLoad(EPWM3_BASE);
  EPWM_setTimeBaseCounter(EPWM1_BASE, 0U);
  EPWM_setSyncOutPulseMode(EPWM1_BASE, EPWM_SYNC_OUT_PULSE_ON_COUNTER_ZERO);

  // Set up counter mode
  EPWM_setTimeBaseCounterMode(EPWM1_BASE, EPWM_COUNTER_MODE_UP);
  EPWM_disablePhaseShiftLoad(EPWM1_BASE);
  EPWM_setClockPrescaler(EPWM1_BASE,
                         EPWM_CLOCK_DIVIDER_1,
                         EPWM_HSCLOCK_DIVIDER_1);

  EPWM_setCounterCompareValue(EPWM1_BASE,
                              EPWM_COUNTER_COMPARE_A,
                              700);

  // Set up shadowing
  EPWM_setCounterCompareShadowLoadMode(EPWM1_BASE,
                                       EPWM_COUNTER_COMPARE_A,
                                       EPWM_COMP_LOAD_ON_CNTR_ZERO);
  EPWM_setCounterCompareShadowLoadMode(EPWM1_BASE,
                                       EPWM_COUNTER_COMPARE_B,
                                       EPWM_COMP_LOAD_ON_CNTR_ZERO);

  // Set Compare values
  EPWM_setCounterCompareValue(EPWM1_BASE,
                                EPWM_COUNTER_COMPARE_A,
                                700);

  EPWM_setCounterCompareValue(EPWM1_BASE,
                                EPWM_COUNTER_COMPARE_C,
                                800);

  // Set Deadband
  EPWM_setDeadBandDelayPolarity(EPWM1_BASE, EPWM_DB_FED, EPWM_DB_POLARITY_ACTIVE_LOW);
  EPWM_setDeadBandDelayMode(EPWM1_BASE, EPWM_DB_RED, true);
  EPWM_setDeadBandDelayMode(EPWM1_BASE, EPWM_DB_FED, true);
  EPWM_setRisingEdgeDelayCount(EPWM1_BASE, PWM_DB);
  EPWM_setFallingEdgeDelayCount(EPWM1_BASE, PWM_DB);


  // Set actions
  EPWM_setActionQualifierAction(EPWM1_BASE,
                                EPWM_AQ_OUTPUT_A,
                                EPWM_AQ_OUTPUT_HIGH,
                                EPWM_AQ_OUTPUT_ON_TIMEBASE_ZERO);
  EPWM_setActionQualifierAction(EPWM1_BASE,
                                EPWM_AQ_OUTPUT_A,
                                EPWM_AQ_OUTPUT_LOW,
                                EPWM_AQ_OUTPUT_ON_TIMEBASE_UP_CMPA);


  // Set Digital Compare
  // TRIP4 -> DCBH
  EPWM_enableDigitalCompareTripCombinationInput(EPWM1_BASE,
                                                EPWM_DC_COMBINATIONAL_TRIPIN4,
                                                EPWM_DC_TYPE_DCBH);
  // DCBH -> DCBEVT2
  EPWM_setTripZoneDigitalCompareEventCondition(EPWM1_BASE,
                                               EPWM_TZ_DC_OUTPUT_B2,
                                               EPWM_TZ_EVENT_DCXH_HIGH);

  // Enable DCB as T1 Source
  EPWM_setActionQualifierT1TriggerSource(EPWM1_BASE, EPWM_AQ_TRIGGER_EVENT_TRIG_DCB_2);

  EPWM_setActionQualifierAction(EPWM1_BASE,
                                EPWM_AQ_OUTPUT_A,
                                EPWM_AQ_OUTPUT_LOW,
                                EPWM_AQ_OUTPUT_ON_T1_COUNT_UP);

  EPWM_enableInterrupt(EPWM1_BASE);
  EPWM_setInterruptSource(EPWM1_BASE, EPWM_INT_TBCTR_U_CMPC);
  Interrupt_enable(INT_EPWM1);
  Interrupt_register(INT_EPWM1, &ISRPreControlTask);
  EPWM_setInterruptEventCount(EPWM1_BASE, 1);
  EPWM_clearEventTriggerInterruptFlag(EPWM1_BASE);
  Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP3);
}

static inline void InitCMP1(void)
{
  CMPSS_enableModule(CMPSS1_BASE);
  CMPSS_configHighComparator(CMPSS1_BASE, CMPSS_INSRC_DAC);
  CMPSS_configDAC(CMPSS1_BASE, CMPSS_DACVAL_PWMSYNC |
                               CMPSS_DACREF_VDDA |
                               CMPSS_DACSRC_RAMP);
  CMPSS_configOutputsHigh(CMPSS1_BASE, CMPSS_TRIP_FILTER |
                                         CMPSS_TRIPOUT_SYNC_COMP);
  CMPSS_setRampDecValue(CMPSS1_BASE, RAMP_DECREMENT);
  CMPSS_setRampDelayValue(CMPSS1_BASE, RAMP_DELAY);
  CMPSS_configRamp(CMPSS1_BASE,
                   27000,
                   RAMP_DECREMENT,
                   RAMP_DELAY,
                   1,
                   false);
  CMPSS_configFilterHigh(CMPSS1_BASE, 2, 20, 15);
  CMPSS_initFilterHigh(CMPSS1_BASE);

  GPIO_setMasterCore(11, GPIO_CORE_CPU1);
  GPIO_setPinConfig(GPIO_11_OUTPUTXBAR7);
  GPIO_setDirectionMode(11, GPIO_DIR_MODE_OUT);
  GPIO_setPadConfig(11, GPIO_PIN_TYPE_STD);

  XBAR_setOutputMuxConfig(XBAR_OUTPUT7, XBAR_OUT_MUX00_CMPSS1_CTRIPOUTH);
  XBAR_enableOutputMux(XBAR_OUTPUT7, XBAR_MUX00);

  XBAR_setEPWMMuxConfig(XBAR_TRIP4, XBAR_EPWM_MUX00_CMPSS1_CTRIPH);
  XBAR_enableEPWMMux(XBAR_TRIP4, XBAR_MUX00);
}

__interrupt void ISRPreControlTask(void)
{

  rampstart = __fmin(rampstart + DVID, __fmax(rampstart,rampstartref));
  rampstart = __fmax(rampstart - DVID, __fmin(rampstart,rampstartref));

  CMPSS_setMaxRampValue(CMPSS1_BASE, (uint32_t)rampstart);

  EPWM_clearEventTriggerInterruptFlag(EPWM1_BASE);
  Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP3);
}
